function [phase,decomp_mu,decomp_cov,AIC_osc,osc_param] = osc_decomp(y,fs,MAX_OSC,MAX_AR)
%
% Input:
%    y:             time series (1 times T)
%    fs:            sampling frequency (scalar)
%    MAX_OSC:       maximum number of oscillation components (scalar)
%    MAX_AR:        maximum AR degree (scalar)
%                   (default value = 2*MAX_OSC)
%
% Output:
%    phase:         estimated phase of each oscillation component (MAX_OSC times T times MAX_OSC)
%                   phase(k,:,K) is the estimated phase of the k-th
%                   oscillator in the decomposition into K components
%    decomp_mu:     smoothed coordinate of each oscillator (2*MAX_OSC times T times MAX_OSC)
%                   decomp_mu(2*k-1:2*k,:,K) is the smoothed coordinate of the k-th oscillator in the decomposition into K components
%    decomp_cov:    smoothed covariance of each oscillator (2*MAX_OSC times 2*MAX_OSC times T times MAX_OSC)
%                   decomp_cov(2*k-1:2*k,2*k-1:2*k,:,K) is the smoothed covariance of the k-th oscillator in the decomposition into K components
%    AIC_osc:       AIC of the oscillator model (1 times MAX_OSC)
%                   AIC_osc(K) is the AIC of the oscillator model with K oscillation components
%    osc_param:     estimated parameters of the oscillator model (MAX_OSC times 3*MAX_OSC+1)
%                   osc_param(K,1:K) is the estimated a_1,...,a_K of the oscillator model with K oscillation components
%                   osc_param(K,K+1:2*K) is the estimated f_1,...f_K of the oscillator model with K oscillation components
%                   osc_param(K,2*K+1:3*K) is the estimated sigma_1^2,...,sigma_K^2 of the oscillator model with K oscillation components
%                   osc_param(K,3*K+1) is the estimated tau^2 of the oscillator model with K oscillation components
%
    options = optimoptions('fminunc','Algorithm','quasi-newton','MaxFunEvals',10000);
    T = length(y);
    phase = zeros(MAX_OSC,T,MAX_OSC);
    decomp_mu = zeros(2*MAX_OSC,T,MAX_OSC);
    decomp_cov = zeros(2*MAX_OSC,2*MAX_OSC,T,MAX_OSC);
    AIC_osc = zeros(1,MAX_OSC);
    osc_param = zeros(MAX_OSC,3*MAX_OSC+1);
    MAX_AR = max(MAX_AR,2*MAX_OSC);
    [ARwithnoise_param,AIC_ARwithnoise] = AR_fit(y,MAX_AR);
    for K=1:MAX_OSC
        K
        minAIC = inf;
        minAIC2 = inf;
        for ARdeg=K:2*K
            tmp = roots([1 ARwithnoise_param(ARdeg,1:ARdeg)]);
            if ARdeg-nnz(imag(tmp))/2 == K && AIC_ARwithnoise(ARdeg) < minAIC
                z0 = tmp;
                E0 = ARwithnoise_param(ARdeg,ARdeg+1);
                R0 = ARwithnoise_param(ARdeg,ARdeg+2);
                minAIC = AIC_ARwithnoise(ARdeg);
                optARdeg = ARdeg;
            end
            if ARdeg-nnz(imag(tmp))/2 >= K && AIC_ARwithnoise(ARdeg) < minAIC2
                z1 = tmp;
                E1 = ARwithnoise_param(ARdeg,ARdeg+1);
                R1 = ARwithnoise_param(ARdeg,ARdeg+2);
                minAIC2 = AIC_ARwithnoise(ARdeg);
                optARdeg2 = ARdeg;
            end
        end
        if minAIC == inf
            if minAIC2 == inf
                warning('no AR model with %d oscillators',K);
            end
            z0 = z1;
            E0 = E1;
            R0 = R1;
            optARdeg = optARdeg2;
            [B,I] = sort(abs(z0),'descend');
            z0 = z0(I);
        end
        [tmp,ARdeg] = min(AIC_ARwithnoise);
        tmp = roots([1 ARwithnoise_param(ARdeg,1:ARdeg)]);
        if nnz(imag(tmp)>=0) >= K
            z0 = tmp;
            E0 = ARwithnoise_param(ARdeg,ARdeg+1);
            R0 = ARwithnoise_param(ARdeg,ARdeg+2);
            optARdeg = ARdeg;
        end
        VV = zeros(optARdeg,optARdeg);
        for j=1:optARdeg
            for i=1:optARdeg
                VV(i,j) = z0(j)^(1-i);
            end
        end
        QQ = inv(VV)*[E0 zeros(1,optARdeg-1); zeros(optARdeg-1,optARdeg)]*inv(VV)';
        [B,I] = sort(diag(real(QQ))./(1-abs(z0).^2),'descend');
        z0 = z0(I);
        
        init_a = zeros(1,K);
        init_f = zeros(1,K);
        kk = 1;
        for k=1:K
            init_a(k) = abs(z0(kk));
            init_f(k) = abs(angle(z0(kk)));
	        if isreal(z0(kk)) && z0(kk) < 0
        	    init_f(k) = 4; % for pi
	        end
            if imag(z0(kk)) == 0
                kk = kk+1;
            else
                kk = kk+2;
            end
        end
        [B,I] = sort(init_f);
        init_a = init_a(I);
        init_f = init_f(I);
%    	nf0 = nnz(init_f==0);
%    	nf1 = nnz(init_f==4);
%	    freq = init_f;
%    	freq(2:nf0) = pi*rand(1,nf0-1);
%	    freq(num_component-nf1+2:num_component) = pi*rand(1,nf1-1);
        if mod(T,2) == 0
        	freq = [2*pi/T*(0:T/2-1) pi];
	    else
    	    freq = [2*pi/T*(0:(T-1)/2)];
        end
        init_f(init_f==4) = pi;
        P = zeros(length(freq),K);
        for k=1:K
            a = init_a(k);
            theta = init_f(k);
            A = (1-2*a^2*cos(theta)^2+a^4*cos(2*theta))/a/(a^2-1)/cos(theta);
            b = (A-2*a*cos(theta)+sign(cos(theta))*sqrt((A-2*a*cos(theta))^2-4))/2;
            for j=1:length(freq)
                P(j,k) = -a*cos(theta)/b*abs(1+b*exp(-1i*freq(j)))^2/abs(1-2*a*cos(theta)*exp(-1i*freq(j))+a^2*exp(-2*1i*freq(j))).^2;
            end
        end
        p = zeros(length(freq),1);
        for j=1:length(freq)
            p(j) = abs(y*exp(-1i*freq(j)*(0:T-1)'))^2/T;
        end
        if cond(P'*P) < 10^6    % modified 2017/01/14
            init_sigma = (P'*P)\(P'*p);
        else
            init_sigma = expGLMfit(P,p);
        end
        init_sigma(init_sigma<0) = R0;
        
        param = fminunc(@(param)osc_ll(y,param,init_f),[atanh(2*init_a-1) zeros(1,K) log(init_sigma'/R0)],options);
        [mll,sigman] = osc_ll(y,param,init_f);
        param(K+1:2*K) = abs(init_f+tanh(param(K+1:2*K))*pi);
        AIC_osc(K) = 2*mll+2*(3*K+1);
	    [tmp,I] = sort(param(K+1:2*K));
        a = (tanh(param(I))+1)/2;
        theta = param(K+I);
        sigma = exp(param(2*K+I))*sigman;
        osc_param(K,1:3*K+1) = [a theta*fs/2/pi sigma sigman];
    
        m = 2*K;
        x_pred1 = zeros(m,T);
        x_filt = zeros(m,T);
        x_smooth = zeros(m,T);
        V_pred1 = zeros(m,m,T);
        V_filt = zeros(m,m,T);
        V_smooth = zeros(m,m,T);
        F = zeros(m,m);
        Q = zeros(m,m);
        H = zeros(1,m);
        for k=1:K
            F(2*k-1:2*k,2*k-1:2*k) = a(k)*[cos(theta(k)) -sin(theta(k)); sin(theta(k)) cos(theta(k))];
            Q(2*k-1:2*k,2*k-1:2*k) = sigma(k)*eye(2);
        end
        H(1:2:2*K) = 1;
        R = sigman;
        x_pred1(:,1) = zeros(m,1);
        for k=1:K
            V_pred1(2*k-1:2*k,2*k-1:2*k,1) = eye(2);
        end
        for t=1:T-1
            Kg = V_pred1(:,:,t)*H'*inv(H*V_pred1(:,:,t)*H'+R);
            x_filt(:,t) = x_pred1(:,t) + Kg*(y(t)-H*x_pred1(:,t));
            V_filt(:,:,t) = (eye(m)-Kg*H)*V_pred1(:,:,t);
            x_pred1(:,t+1) = F*x_filt(:,t);
            V_pred1(:,:,t+1) = F*V_filt(:,:,t)*F'+Q;
        end
        Kg = V_pred1(:,:,T)*H'*inv(H*V_pred1(:,:,T)*H'+R);
        x_filt(:,T) = x_pred1(:,T) + Kg*(y(T)-H*x_pred1(:,T));
        V_filt(:,:,T) = (eye(m)-Kg*H)*V_pred1(:,:,T);
        x_smooth(:,T) = x_filt(:,T);
        V_smooth(:,:,T) = V_filt(:,:,T);
        for t=T-1:-1:1
            x_smooth(:,t) = x_filt(:,t) + V_filt(:,:,t)*F'*(V_pred1(:,:,t+1)\(x_smooth(:,t+1)-x_pred1(:,t+1)));
            CC = V_filt(:,:,t)*F'*inv(V_pred1(:,:,t+1));
            V_smooth(:,:,t) = V_filt(:,:,t) + CC*(V_smooth(:,:,t+1)-V_pred1(:,:,t+1))*CC';
        end
        for k=1:K
            phase(k,:,K) = atan2(x_smooth(2*k,:),x_smooth(2*k-1,:));
        end
        decomp_mu(1:m,:,K) = x_smooth;
        decomp_cov(1:m,1:m,:,K) = V_smooth;
    end
end


